<?php

namespace Drupal\role_toggle;

use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\User;
use Drupal\user\RoleInterface;

class RoleToggle {

  public static function permissions() {
    $perms = array();
    foreach (self::roles() as $role) {
      $perms[self::permissionId($role)] = array(
        'title' => self::permissionLabel($role),
      );
    }
    return $perms;
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function userAccess($role) {
    return \Drupal::currentUser()->hasPermission(self::permissionId($role));
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return string
   */
  public static function permissionId($role) {
    return 'role_toggle:' . $role->id();
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public static function permissionLabel($role) {
    return t('Toggle role %role', array('%role' => $role->label()));
  }

  /**
   * Get all applicable user roles.
   *
   * @return \Drupal\user\RoleInterface[]
   */
  public static function roles() {
    $roles = user_roles();
    unset($roles[\Drupal\Core\Session\AccountInterface::ANONYMOUS_ROLE]);
    unset($roles[\Drupal\Core\Session\AccountInterface::AUTHENTICATED_ROLE]);
    return $roles;
  }

  /**
   * @return \Drupal\user\RoleInterface[]
   */
  public static function togglableRoles() {
    return array_filter(self::roles(), function(RoleInterface $role) {
      return self::canToggle($role);
    });
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function canToggle($role, AccountInterface $account = NULL) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }
    return $account->isAuthenticated()
      && $account->hasPermission(self::permissionId($role));
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface|null $account
   * @return bool
   */
  public static function canToggleAny(AccountInterface $account = NULL) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }
    foreach (self::roles() as $role) {
      if (self::canToggle($role, $account)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function isEnabledRole($role) {
    return in_array($role->id(), \Drupal::currentUser()->getRoles());
  }

  public static function indicatorClass($enabled) {
    return "role-toggle-role-" . ($enabled ? 'active' : 'inactive');
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function toggleRole($role) {
    $rid = $role->id();
    $currentUser = User::load(\Drupal::currentUser()->id());
    $hasRole = $currentUser->hasRole($rid);
    if ($hasRole) {
      $currentUser->removeRole($rid);
    }
    else {
      $currentUser->addRole($rid);
    }
    $currentUser->save();
    return !$hasRole;
  }

}
